

function teamsJsClient(tabUrl, allowTopNavigation, whitelistedOrigin) {

 /* Validate tabUrl, must be  valid https URL */
  if (typeof tabUrl !== 'string' || !tabUrl) {
    console.error('Tab url is invalid.');
    return;
  }

  if (!tabUrl.match('^https')) {
    console.error('Tab url must use HTTPS.');
    return;
  }

  /* Error strings */
  const errorStrings = {
    nativeInterfaceUndefined: 'Couldn\'t find a native interface to delegate the method to.',
    tabFrameUndefined: 'Couldn\'t find a tab frame to host the tab.',
    unhandledSDkRequest: 'Received a request that is not handled. Request: ',
    failedToGetAuthToken: 'Failed to get auth token.',
    failedToGetUser: 'Failed to get user.',
    resourceNotSpecifiedForAuth: 'Resource url not specified for authentication.getAuthToken request.',
    filePreviewArgsInvalid: 'Invalid arguments for openFilePreview message.'
  };

  /* Validate tab frame exists */
  var tabFrame = document.getElementById('extension-tab-frame');
  var tabFrameWindow = tabFrame.contentWindow;

  if (!tabFrame || !tabFrameWindow) {
    console.error(errorStrings.tabFrameUndefined);
    return;
  }

  /* Attach a listener for message events */
  window.addEventListener('message', function (event) { processMessage(event) }, false);
  console.debug("Attached listener for message events.");

  /* Update sandbox restrictions */
  if (allowTopNavigation && tabFrame.sandbox && !tabFrame.sandbox.contains("allow-top-navigation-by-user-activation")) {
    tabFrame.sandbox.add("allow-top-navigation-by-user-activation");
  }

  /* Update the tab iframe src */
  tabFrame.src = tabUrl;

    /* Processes the message recevied by the document window */
    function processMessage(event) {
     if (event.origin === 'https://teams.microsoft.com') {
        var eventId = event.data.eventId;
        var response = {};
          if (eventId === 'initialize') {
            handleInitializeConnection(event);
          } else if (eventId === 'metadata' && ensureNativeInterface()) {
            handleMetadataEvent(event);
          } else if (eventId === "heartbeat") {
            handleHeartBeatEvent(event);
          } else if (event.data.payload != undefined) {
              if (event.data.payload.eventId === 'aadCurrentUser') {
                handleAadCurrentUserEvent(event);
              } else if (event.data.payload.eventId === 'aadGetToken') {
                handleGetAadTokenEvent(event);
              }
          }
          console.log(event);
        }
    }

    function handleInitializeConnection(event) {
        console.debug('handling initialize');
        response = {};
        response.eventId = 'initialize';
        response.correlationId = event.data.correlationId;
        response.payload = {result: true};
        var uiLanguagePreference = null;
        if (ensureNativeInterface()) {
          NativeInterface.onInitializeStreamConnectionReceived();
          uiLanguagePreference = NativeInterface.getUiLanguagePreference();
        }
        if (uiLanguagePreference != null) {
            response.payload.uiLanguagePreference = uiLanguagePreference;
        }
        if (whitelistedOrigin) {
            tabFrame.contentWindow.postMessage(response, whitelistedOrigin);
        } else {
            tabFrame.contentWindow.postMessage(response, event.origin);
        }
    }

    function handleMetadataEvent(event) {
        console.debug('handling metadata event');
        var metadata = event.data.payload;
        if (metadata != null) {
            NativeInterface.qnaUrlReceived(metadata.qnaUrl);
            NativeInterface.meetingInfoReceived(metadata.supportUrl, metadata.title, metadata.utcStart, metadata.utcEnd, event.data.correlationId);
            if (metadata.streamUrl != null && metadata.streamUrl != "undefined") {
                NativeInterface.streamUrlReceived(metadata.streamUrl);
            }
            if (metadata.yammerBroadcastId != null && metadata.yammerBroadcastId != "undefined") {
                NativeInterface.yammerBroadcastIdReceived(metadata.yammerBroadcastId);
            }
        }
    }

    function handleAadCurrentUserEvent(event) {
        console.debug("handling aadCurrentUser request");
        var response = {};
        if (ensureNativeInterface()) {
            var obj = JSON.parse(NativeInterface.getAadCurrentUser());
            response.payload = {authUser: { isAuthenticated: true, profile: obj}};
        }
        response.eventId = event.data.payload.eventId;
        response.correlationId = event.data.payload.correlationId;
        response.channel = event.data.channel;
        response.metaData = {correlationId : event.data.payload.correlationId, destination: {channel: event.data.payload.eventId, processIds: [] }};
         if (whitelistedOrigin) {
            tabFrame.contentWindow.postMessage(response, whitelistedOrigin);
         } else {
            tabFrame.contentWindow.postMessage(response, event.origin);
         }
    }

    function handleGetAadTokenEvent(event) {
        console.debug("handling handleGetAadTokenEvent request");
        var response = {};
        response.eventId = event.data.payload.eventId;
        response.correlationId = event.data.payload.correlationId;
        if (ensureNativeInterface()) {
            var obj = JSON.parse(NativeInterface.getAuthToken(event.data.payload.payload.resource));
            response.payload = {token: obj.token, expires: obj.expires};
        }
        response.channel = event.data.channel;
        response.metaData = {correlationId : event.data.payload.correlationId, destination: {channel: event.data.payload.eventId, processIds: [] }};
         if (whitelistedOrigin) {
            tabFrame.contentWindow.postMessage(response, whitelistedOrigin);
         } else {
            tabFrame.contentWindow.postMessage(response, event.origin);
         }
    }

    function handleHeartBeatEvent(event) {
        console.debug("handling heartbeat event");
        if (ensureNativeInterface()) {
            NativeInterface.handleCanPlayThrough();
        }
    }

    /* Ensures the native JS interface is defined */
    function ensureNativeInterface() {
        if (typeof NativeInterface === 'undefined') {
            console.error(errorStrings.nativeInterfaceUndefined);
            return false;
        }
        return true;
    }
}